/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : ConsoleProgressBar
   date       : 13.03.98
   description: a progress bar in the console window

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

//import java.io.*;
//import java.util.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class ConsoleProgressBar {

  // class variable declarations
  private final static char DEF_FULL_CHAR  = ':';
  private final static char DEF_EMPTY_CHAR = '.';
  private final static int  DEF_MAX        = 10;

  // instance variable declarations
  private String label;
  private char cFull;
  private char cEmpty;
  private int cur;
  private int max;

  // constructors
  public ConsoleProgressBar() {
    init("",DEF_FULL_CHAR,DEF_EMPTY_CHAR,0,DEF_MAX);
  } // end constructor ConsoleProgressBar

  public ConsoleProgressBar(String label) {
    init(label,DEF_FULL_CHAR,DEF_EMPTY_CHAR,0,DEF_MAX);
  } // end constructor ConsoleProgressBar

  public ConsoleProgressBar(String label, int max) {
    init(label,DEF_FULL_CHAR,DEF_EMPTY_CHAR,0,max);
  } // end constructor ConsoleProgressBar

  public ConsoleProgressBar(String label, char cFull, char cEmpty,
                            int cur, int max) {
    init(label,DEF_FULL_CHAR,DEF_EMPTY_CHAR,0,max);
  } // end constructor ConsoleProgressBar

  // methods
  private void init(String label, char cFull, char cEmpty, int cur,
                    int max) {
    int i;
    String str = "";

    this.label  = label;
    this.cFull  = cFull;
    this.cEmpty = cEmpty;
    this.cur    = cur;
    this.max    = max;
    System.out.print(label);
    if (cur > max) cur = max;
    for (i=0; i<max; i++) str += cEmpty;
    for (i=0; i<max; i++) str += '\u0008';
    System.out.print(str);
    inc(cur);
  } // end init

  public void inc(int diff) {
    int i;
    String str = "";

    if (cur+diff > max) diff = max-cur;
    for (i=0; i<diff; i++) str += cFull;
    cur += diff;
    System.out.print(str);
    System.out.flush();
  } // end inc

  public int get() {
    return cur;
  } // end get

  public void dispose() {
    int n = label.length()+cur;
    int i;
    String str = "";

    for (i=0; i<n; i++) str += '\u0008';
    n = n-cur+max;
    for (i=0; i<n; i++) str += ' ';
    for (i=0; i<n; i++) str += '\u0008';
    System.out.print(str);
    System.out.flush();
  } // end dispose

} // end ConsoleProgressBar

/* ******************************************************************
   end of file
*********************************************************************/
