/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac             

   class      : DateChooserDlg
   date       : 28.11.97
   description: a dialog that allows you to choose a date
   modified   : 02.02.98 - auto cancel mode added
                           auto cancel on window deactivation is
                           initially enabled, because DateChooserDlg is
                           often used as a popup dialog executed by
                           controls
                09.02.98 - new constructors for locale scheme added
                13.03.98 - dialog is no longer initially visible

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class DateChooserDlg extends Dialog implements ItemListener {

  // instance variable declarations
  private Calendar calendar;
  private boolean canceled   = false;
  private boolean autoCancel = true;
  private boolean done       = false;
  DateChooserPanel dayChooser;

  // inner classes
  class DateChooserDlgListener extends WindowAdapter {

    // methods
    public void windowClosing(WindowEvent e) {
      setCanceled(true);
      setVisible(false);
      dispose();
    } // end windowClosing

    public void windowDeactivated(WindowEvent e) {
      if (autoCancel) {
        setCanceled(true);
        setVisible(false);
        dispose();
      } // end if
    } // end WindowDeactivated

  } // end DateChooserDlgListener

  // constructors
  public DateChooserDlg(Frame parent, Calendar calendar) {
    super(parent);
    init(calendar,Locale.GERMAN);
  } // end constructor DateChooserDlg

  public DateChooserDlg(Frame parent, boolean modal, Calendar calendar) {
    super(parent,modal);
    init(calendar,Locale.GERMAN);
  } // end constructor DateChooserDlg

  public DateChooserDlg(Frame parent, String title, Calendar calendar) {
    super(parent,title);
    init(calendar,Locale.GERMAN);
  } // end constructor DateChooserDlg

  public DateChooserDlg(Frame parent, String title,
                         boolean modal, Calendar calendar) {
    super(parent,title,modal);
    init(calendar,Locale.GERMAN);
  } // end constructor DateChooserDlg

  public DateChooserDlg(Frame parent, String title,
                         boolean modal, Calendar calendar, Locale locale) {
    super(parent,title,modal);
    init(calendar,locale);
  } // end constructor DateChooserDlg

  // methods
  private void init(Calendar calendar, Locale locale) {
    addWindowListener(new DateChooserDlgListener());
    this.calendar = (Calendar)calendar.clone();
    
    dayChooser = new DateChooserPanel(calendar,locale);
    dayChooser.addItemListener(this);
    add(dayChooser);

    pack();
    setResizable(false);
    //setVisible(true);
  } // end init

  public void itemStateChanged(ItemEvent e) {
    calendar.setTime(dayChooser.getDate());
    if (dayChooser.getMouseSelected()) {
      done = true;
      setVisible(false);
      dispose();
    } // end if
  } // end itemStateChanged

  public Date getDate() {
    return calendar.getTime();
  } // end getDate

  public void setDate(Date date) {
    calendar.setTime(date);
    dayChooser.setDate(calendar.getTime());
  } // end getDate

  public boolean getCanceled() {
    return canceled && !done;
  } // end getCanceled

  public void setCanceled(boolean canceled) {
    this.canceled = canceled;
  } // end setCanceled

  public boolean getAutoCancel() {
    return autoCancel;
  } // end getAutoCancel

  public void setAutoCancel(boolean autoCancel) {
    this.autoCancel = autoCancel;
  } // end getAutoCancel

} // end DateChooserDlg

/* ******************************************************************
   end of file
*********************************************************************/
