/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : DeleteSelDlg
   date       : 11.03.98
   description: a dialog that displays dates added or modified by
                other users
   modified   : 13.03.98 - dialog is no longer initially visible

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/


import java.awt.*; 
import java.awt.event.*;
import java.util.*;
import java.text.SimpleDateFormat;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class DeleteSelDlg extends Dialog implements ActionListener {

  // instance variable declarations
  private Frame parent;
  private boolean canceled    = false;
  private boolean autoCancel  = false;
  private boolean autoDisable = false;
  private Button delButton;
  private Button viewButton;
  private Panel cbPanel;
  private CheckboxGroup cbg;
  private Checkbox entry1Checkbox;
  private Checkbox entry2Checkbox;
  private CalendarEntry entry1;
  private CalendarEntry entry2;

  // inner classes
  class DeleteSelDlgListener extends WindowAdapter {

    // methods
    public void windowClosing(WindowEvent e) {
      setCanceled(true);
      setVisible(false);
      dispose();
    } // end windowClosing

    public void windowActivated(WindowEvent e) {
    } // end WindowActivated

    public void windowDeactivated(WindowEvent e) {
      if (autoCancel) {
        setCanceled(true);
        setVisible(false);
        dispose();
      } // end if
    } // end WindowDeactivated

  } // end DeleteSelDlgListener

  class EditDlgListener extends WindowAdapter {

    public void windowClosed(WindowEvent e) {
      autoCancel  = true;
      autoDisable = false;
    } // end windowClosed
  
  } // end EditDlgListener

  // constructors
  public DeleteSelDlg(Frame parent, CalendarEntry entry1,
                      CalendarEntry entry2) {
    super(parent);
    this.parent       = parent;
    this.entry1       = entry1;
    this.entry2       = entry2;
    init();
  } // end constructor DeleteSelDlg

  public DeleteSelDlg(Frame parent, boolean modal, CalendarEntry entry1,
                      CalendarEntry entry2) {
    super(parent,modal);
    this.parent       = parent;
    this.entry1       = entry1;
    this.entry2       = entry2;
    init();
  } // end constructor DeleteSelDlg

  public DeleteSelDlg(Frame parent, String title, CalendarEntry entry1,
                      CalendarEntry entry2) {
    super(parent,title);
    this.parent       = parent;
    this.entry1       = entry1;
    this.entry2       = entry2;
    init();
  } // end constructor DeleteSelDlg

  public DeleteSelDlg(Frame parent, String title, boolean modal,
                      CalendarEntry entry1, CalendarEntry entry2) {
    super(parent,title,modal);
    this.parent       = parent;
    this.entry1       = entry1;
    this.entry2       = entry2;
    init();
  } // end constructor DeleteSelDlg

  // methods
  private void init() {
    addWindowListener(new DeleteSelDlgListener());
    // this dialog box uses the gridbaglayout to provide a pretty good layout
    GridBagLayout gridbag          = new GridBagLayout(); 
    GridBagConstraints constraints = new GridBagConstraints();

    // create the widgets
    InsetsPanel pan = new InsetsPanel(new Insets(15,20,15,20));
    delButton = new Button("Delete");
    delButton.addActionListener(this);
    viewButton = new Button("View");
    viewButton.addActionListener(this);
    InsetsPanel buttonPanel = new InsetsPanel(new Insets(15,0,0,0));
    buttonPanel.setLayout(new GridLayout(1,2,10,0));
    buttonPanel.add(delButton);
    buttonPanel.add(viewButton);

    // set up the checkboxes
    {
      SimpleDateFormat formatter = new SimpleDateFormat("d.MM.yy H:mm, ",
                                         CalendarCreator.getLocale());
      String str;

      cbPanel = new Panel(new GridLayout(2,1));

      cbg = new CheckboxGroup();
      str = formatter.format(entry1.getStart())+
            entry1.getCreator() + ": "+
            entry1.getDescription();
      entry1Checkbox = new Checkbox(str,cbg,true);
      cbPanel.add(entry1Checkbox);
      str = formatter.format(entry2.getStart())+
            entry2.getCreator() + ": "+
            entry2.getDescription();
      entry2Checkbox = new Checkbox(str,cbg,false);
      cbPanel.add(entry2Checkbox);
    }

    Label label = getDescriptionLabel();

    pan.setLayout(gridbag); 

    constraints.fill       = GridBagConstraints.NONE; 
    constraints.anchor     = GridBagConstraints.WEST; 
    constraints.weightx    = 1.0; 
    constraints.weighty    = 1.0; 
    constraints.gridwidth  = GridBagConstraints.REMAINDER; 
    constraints.gridheight = 1; 
    gridbag.setConstraints(label,constraints); 
    pan.add(label); 

    constraints.anchor     = GridBagConstraints.CENTER; 
    constraints.gridheight = GridBagConstraints.RELATIVE; 
    gridbag.setConstraints(cbPanel,constraints); 
    pan.add(cbPanel); 

    constraints.gridheight = GridBagConstraints.REMAINDER; 
    gridbag.setConstraints(buttonPanel, constraints); 
    pan.add(buttonPanel); 

    add(pan);
    pack(); 
    setResizable(false);
    //setVisible(true);
  } // end init

  protected Label getDescriptionLabel() {
    return new Label("The following dates are overlapping - select "+
                     " one to delete:");
  } // end getDescriptionLabel

  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();

    if (cmd.equals("View")) {
      CalendarEntry entry = entry1Checkbox.getState() ? entry1 : entry2;
      EditDateDlg editDlg = new EditDateDlg(parent,"View Date",isModal(),
                                            entry,EditDateDlg.VIEW,
                                            entry.getPersons());
      if (autoCancel) {
        autoCancel = false;
        autoDisable = true;
        editDlg.addWindowListener(new EditDlgListener());
        editDlg.setAutoCancel(true);
      } // end if
      editDlg.setVisible(true);
    }
    else { // if cmd equals Delete
      setVisible(false);
      dispose();
    } // end if
  } // end actionPerformed

  public CalendarEntry getSelectedEntry() {
    if (entry1Checkbox.getState()) {
      return entry1;
    }
    else {
      Assert.assert(entry2Checkbox.getState());
      return entry2;
    } // end if
  } // end getSelectedEntry

  public boolean getCanceled() {
    return canceled;
  } // end getCanceled

  public void setCanceled(boolean canceled) {
    this.canceled = canceled;
  } // end getCanceled

  public boolean getAutoCancel() {
    return autoCancel;
  } // end getAutoCancel

  public void setAutoCancel(boolean autoCancel) {
    this.autoCancel = autoCancel;
  } // end getAutoCancel

} // end DeleteSelDlg

/* ******************************************************************
   end of file
*********************************************************************/
