/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : JavaNotesGUI
   date       : 03.01.98
   description: the graphical user interface for the JavaNotes
                project
   modified   : 18.02.98 - method updateNow added
                19.02.98 - monthly panel installed
                13.03.98 - most of the dialogs are set to visible
                           manually due to a bug while running
                           JavaNotes at Solaris
                19.03.98 - bug dealing around with CardLayout removed

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class JavaNotesGUI extends Panel
  implements ActionListener, ItemListener, Observer {

  // class variable declarations
  private final static long GETMESSAGES_TIME = 15000;      // milliseconds
  private final static String DAYLY_PANEL    = "d";
  private final static String WEEKLY_PANEL   = "w";
  private final static String MONTHLY_PANEL  = "m";

  // instance variable declarations
  private CalendarDatabase db;
  private Frame parent;
  private boolean inApplet;
  private int vSize;
  private String helpDoc;
  private MemoryImageCanvas dukeIcon;
  private LoginInfoModel lModel;
  private ScopeModel sModel;
  private DateListModel dModel;
  private DateListRefresher dRefresh;
  private AppletMenuBar menuBar;
  private CheckboxMenuItemGroup cbmGroup;
  private ToolPanel toolPanel;
  private CardLayout cardLayout;
  private Panel cardPanel;
  private Panel daylyPanel;
  private Panel weeklyPanel;
  private Panel monthlyPanel;
  private long lastGetMessagesQuery = 0;

  // inner classes
  class LoginDlgListener extends WindowAdapter {

    private LoginDlg loginDlg;

    public LoginDlgListener(LoginDlg loginDlg) {
      super();
      this.loginDlg = loginDlg;
    } // end constructor LoginDlgListener

    public void windowClosed(WindowEvent e) {
      if (!loginDlg.getCanceled()) {
        String user   = loginDlg.getUser();
        String passwd = loginDlg.getPasswd();
        String errorMsg;
        String persons[];
        db.login(user,passwd);
        errorMsg = db.getErrorMsg();
        if (errorMsg != null) {
          (new ErrorDlg(parent,"Error",!inApplet,errorMsg)).setAutoCancel(inApplet);
        }
        else {
          persons = db.getUsers();
          errorMsg = db.getErrorMsg();
          if (errorMsg != null) {
            (new ErrorDlg(parent,"Error",!inApplet,errorMsg)).setAutoCancel(inApplet);
          }
          else {
            lModel.setUser(user);
            lModel.setPasswd(passwd);
            lModel.setPersons(persons);
            lModel.setLogedIn(true);
          } // end if
        } // end if
      } // end if
    } // end windowClosing

  } // end LoginDlgListener

  class FindDlgListener extends WindowAdapter {

    private FindDateDlg findDlg;

    public FindDlgListener(FindDateDlg findDlg) {
      super();
      this.findDlg = findDlg;
    } // end constructor FindDlgListener

    public void windowClosed(WindowEvent e) {
      if (!findDlg.getCanceled()) {
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        Vector vector = findDlg.getResultVector();
        dRefresh.clipFreeDateVector(vector,findDlg.getPersons(),
                                    findDlg.getHoursReqire(),
                                    findDlg.getMinutesReqire());

        if (vector.size() == 0) {
          MsgBoxDlg msgBoxDlg = new MsgBoxDlg(parent,"Find Result",!inApplet,
                         "No free Time found for your Search Specification",
                         MsgBoxDlg.OK | MsgBoxDlg.ICONINFORMATION);
          msgBoxDlg.setAutoCancel(inApplet);
          msgBoxDlg.setVisible(true);
        }
        else {
          FindResultDlg findResultDlg = new FindResultDlg(parent,
                                              "Find Result",!inApplet,
                                              vector,findDlg.getHoursReqire(),
                                              findDlg.getMinutesReqire(),
                                              findDlg.getCalendarEntry());
          findResultDlg.addWindowListener(new ResultDlgListener(findResultDlg));
          if (inApplet) findResultDlg.setAutoCancel(true);
          findResultDlg.setVisible(true);
        } // end if
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
      } // end if
    } // end windowClosed

  } // end FindDlgListener

  class ResultDlgListener extends WindowAdapter {

    private FindResultDlg resultDlg;

    public ResultDlgListener(FindResultDlg resultDlg) {
      super();
      this.resultDlg = resultDlg;
    } // end constructor ResultDlgListener

    public void windowClosed(WindowEvent e) {
      if (!resultDlg.getCanceled()) {
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

        EditDateDlg editDlg = new EditDateDlg(parent,"New Date",!inApplet,
                                    resultDlg.getCalendarEntry(),
                                    EditDateDlg.FIND_RESULT_NEW,
                                    lModel.getPersons());
        editDlg.addWindowListener(new EditDlgListener(editDlg));
        if (inApplet) editDlg.setAutoCancel(true);
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        editDlg.setVisible(true);
      } // end if
    } // end windowClosed

  } // end ResultDlgListener

  class EditDlgListener extends WindowAdapter {

    private EditDateDlg editDlg;

    public EditDlgListener(EditDateDlg editDlg) {
      super();
      this.editDlg = editDlg;
    } // end constructor EditDlgListener

    public void windowClosed(WindowEvent e) {
      if (!editDlg.getCanceled()) {
        toolPanel.handleEditDlgResult(editDlg,EditDateDlg.FIND_RESULT_NEW);
      } // end if
    } // end windowClosed
  
  } // end EditDlgListener

  class PwdDlgListener extends WindowAdapter {

    private ChangePasswdDlg pwdDlg;

    public PwdDlgListener(ChangePasswdDlg pwdDlg) {
      super();
      this.pwdDlg = pwdDlg;
    } // end constructor PwdDlgListener

    public void windowClosed(WindowEvent e) {
      if (!pwdDlg.getCanceled()) {
        String passwd = pwdDlg.getPasswd();
        String errorMsg;
        db.changePasswd(passwd);
        errorMsg = db.getErrorMsg();
        if (errorMsg != null) {
          (new ErrorDlg(parent,"Error",!inApplet,errorMsg)).setAutoCancel(inApplet);
        }
        else {
          lModel.setPasswd(passwd);
        } // end if
      } // end if
    } // end windowClosing

  } // end PwdDlgListener

  class DelSelDlgListener extends WindowAdapter {

    private DeleteSelDlg delSelDlg;

    public DelSelDlgListener(DeleteSelDlg delSelDlg) {
      super();
      this.delSelDlg = delSelDlg;
    } // end constructor DelSelDlgListener

    public void windowClosed(WindowEvent e) {
      toolPanel.Delete(delSelDlg.getSelectedEntry());
    } // end windowClosing

  } // end DelSelDlgListener

  // constructors
  public JavaNotesGUI(CalendarDatabase db, Frame parent,
                      boolean inApplet, int vSize, String helpDoc) {
    this.db       = db;
    this.parent   = parent;
    this.inApplet = inApplet;
    this.vSize    = vSize;
    this.helpDoc  = helpDoc;
    dukeIcon      = new MemoryImageCanvas(DukeImagePixmap.PIXELS,
                                          DukeImagePixmap.WIDTH,
                                          DukeImagePixmap.HEIGHT);
    lModel        = new LoginInfoModel();
    sModel        = new ScopeModel();
    dModel        = new DateListModel();
    dRefresh      = new DateListRefresher(lModel,sModel,dModel,db);
    dModel.addObserver(this);

    GridBagLayout gridbag     = new GridBagLayout(); 
    GridBagConstraints constr = new GridBagConstraints();

    // set the color
    setBackground(SystemColor.control);

    // create the controls
    menuBar                   = createMenuBar();
    toolPanel                 = new ToolPanel(parent,inApplet,lModel,
                                              sModel,dModel,db);
    SeparatorCanvas separator = new SeparatorCanvas();
    cardPanel                 = createCardPanel();

    // do the layout
    setLayout(gridbag); 

    if (inApplet) {
      Panel menuPanel   = menuBar.getMenuPanel();
      constr.fill       = GridBagConstraints.NONE; 
      constr.anchor     = GridBagConstraints.NORTHWEST; 
      constr.weightx    = 1.0; 
      constr.weighty    = 1.0; 
      constr.gridwidth  = GridBagConstraints.REMAINDER; 
      constr.gridheight = 1; 
      gridbag.setConstraints(menuPanel,constr); 
      add(menuPanel);
    }
    else {
      parent.setMenuBar(menuBar);
    } // end if

    constr.fill       = GridBagConstraints.HORIZONTAL;
    constr.anchor     = GridBagConstraints.NORTH; 
    constr.weightx    = 1.0; 
    constr.weighty    = 1.0; 
    constr.gridwidth  = GridBagConstraints.REMAINDER; 
    constr.gridheight = 1; 
    gridbag.setConstraints(separator,constr); 
    add(separator);

    constr.fill       = GridBagConstraints.NONE; 
    constr.anchor     = GridBagConstraints.NORTHWEST; 
    gridbag.setConstraints(toolPanel,constr); 
    add(toolPanel);

    separator         = new SeparatorCanvas();
    constr.fill       = GridBagConstraints.HORIZONTAL;
    constr.anchor     = GridBagConstraints.NORTH; 
    constr.gridheight = GridBagConstraints.RELATIVE; 
    gridbag.setConstraints(separator,constr); 
    add(separator);

    constr.fill       = GridBagConstraints.NONE;
    constr.anchor     = GridBagConstraints.CENTER;
    constr.gridheight = GridBagConstraints.REMAINDER; 
    gridbag.setConstraints(cardPanel,constr); 
    add(cardPanel);

  } // end constructor JavaNotesGUI

  // methods
  private AppletMenuBar createMenuBar() {
    AppletMenuBar mb = new AppletMenuBar();
    Menu file;
    Menu view;
    Menu options;
    Menu help;
    MenuItem item;
    CheckboxMenuItem cbmItem;

    // the menus
    if (inApplet) {
      file    = new PopupMenu("File");
      view    = new PopupMenu("View");
      options = new PopupMenu("Options");
      help    = new PopupMenu("Help");
    }
    else {
      file    = new Menu("File");
      view    = new Menu("View");
      options = new Menu("Options");
      help    = new Menu("Help");
    } // end if

    // the file menu items
    file.add(item = new MenuItem("Login ..."));
    item.addActionListener(this);
    file.add(item = new MenuItem("Logout"));
    item.addActionListener(this);
    file.add(item = new MenuItem("Get Messages ..."));
    item.addActionListener(this);
    file.addSeparator();
    file.add(item = new MenuItem("Exit"));
    if (inApplet)
      item.setEnabled(false);
    else
      item.addActionListener(this);
    // end if
    mb.add(file);

    // the view menu items
    cbmGroup = new CheckboxMenuItemGroup();
    view.add(cbmItem = new CheckboxMenuItem("Daily"));
    cbmGroup.add(cbmItem);
    cbmItem.addItemListener(this);
    view.add(cbmItem = new CheckboxMenuItem("Weekly"));
    cbmGroup.add(cbmItem);
    cbmItem.addItemListener(this);
    view.add(cbmItem = new CheckboxMenuItem("Monthly"));
    cbmGroup.add(cbmItem);
    cbmItem.addItemListener(this);
    mb.add(view); 

    // the options menu items
    options.add(item = new MenuItem("Change Password ..."));
    item.addActionListener(this);
    options.add(item = new MenuItem("Find free Date ..."));
    item.addActionListener(this);
    mb.add(options);

    // the help menu items
    help.add(item = new MenuItem("Using Java Notes ..."));
    if (helpDoc == null)
      item.setEnabled(false);
    else
      item.addActionListener(this);
    // end if
    help.addSeparator();
    help.add(item = new MenuItem("About ..."));
    item.addActionListener(this);
    mb.add(help);

    return mb;
  } // end createMenuBar

  private Panel createCardPanel() {
    cardLayout = new CardLayout();
    //Panel cp   = new Panel();
    Panel cp   = new Panel(cardLayout);

    daylyPanel = new DaylyPanel(lModel,sModel,dModel,vSize);
    cp.add(DAYLY_PANEL,daylyPanel);
    weeklyPanel = new WeeklyPanel(lModel,sModel,dModel,vSize);
    cp.add(WEEKLY_PANEL,weeklyPanel);
    monthlyPanel = new MonthlyPanel(lModel,sModel,dModel,vSize);
    cp.add(MONTHLY_PANEL,monthlyPanel);
    //cp.setLayout(cardLayout);
    return cp;
  } // end createCardPanel

  public void update(Observable obs, Object arg) {
    String errorMsg = dRefresh.getErrorMsg();

    if (errorMsg != null) {
      (new ErrorDlg(parent,"Error",!inApplet,errorMsg)).setAutoCancel(inApplet);
    }
    else if (dModel.hasOverlappingEntries() && dModel.isNew() &&
             lModel.getUser().equals(sModel.getUser())) {
      Enumeration enum = dModel.getOverlappingEntries();
      DeleteSelDlg delSelDlg = new DeleteSelDlg(parent,"Delete",
                                          !inApplet,
                                          (CalendarEntry)enum.nextElement(),
                                          (CalendarEntry)enum.nextElement());
      if (inApplet) delSelDlg.setAutoCancel(true);
      delSelDlg.addWindowListener(new DelSelDlgListener(delSelDlg));
      delSelDlg.setVisible(true);
      Assert.out.println("JavaNotesGUI: Overlapping entries");
    } // end if
  } // end update

  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();

    if (cmd.equals("Login ...")) {
      LoginDlg loginDlg = new LoginDlg(parent,"Login",!inApplet);
      if (inApplet) loginDlg.setAutoCancel(true);
      loginDlg.addWindowListener(new LoginDlgListener(loginDlg));
      loginDlg.setVisible(true);
    }
    else if (cmd.equals("Logout")) {
      Logout();
    }
    else if (cmd.equals("Get Messages ...")) {
      GetMessages(false);
    }
    else if (cmd.equals("Exit")) {
      parent.dispose();
    }
    else if (cmd.equals("Change Password ...")) {
      ChangePasswdDlg pwdDlg = new ChangePasswdDlg(parent,"Change Password",
                                                   !inApplet,
                                                   lModel.getPasswd());
      if (inApplet) pwdDlg.setAutoCancel(true);
      pwdDlg.addWindowListener(new PwdDlgListener(pwdDlg));
      pwdDlg.setVisible(true);
    }
    else if (cmd.equals("Find free Date ...")) {
      FindDateDlg findDateDlg = new FindDateDlg(parent,"Find free Date",
                                          !inApplet,lModel.getPersons(),
                                          lModel.getUser());
      if (inApplet) findDateDlg.setAutoCancel(true);
      findDateDlg.addWindowListener(new FindDlgListener(findDateDlg));
      findDateDlg.setVisible(true);
    }
    else if (cmd.equals("Using Java Notes ...")) {
      if (inApplet) {
        try {
          Applet applet = (Applet)this.getParent();
          applet.getAppletContext().showDocument(new java.net.URL(applet.getCodeBase(),helpDoc),
                                                 "Using Java Notes");
        } catch (Exception excep) {
          String errorMsg = "Error on showing document '"+helpDoc+"'\n"+excep;
          (new ErrorDlg(parent,"Error",!inApplet,errorMsg)).setAutoCancel(inApplet);
        }
      }
      else {
        Runtime r = Runtime.getRuntime();
        try {
          r.exec(helpDoc);
        } catch (java.io.IOException excep) {
          String errorMsg = "Error on executing '"+helpDoc+"'\n"+excep;
          (new ErrorDlg(parent,"Error",!inApplet,errorMsg)).setAutoCancel(inApplet);
        }
      } // end if
    }
    else if (cmd.equals("About ...")) {
      MsgBoxDlg msgBoxDlg = new MsgBoxDlg(parent,"About",!inApplet,
                     "JavaNotes 1.0.1\n(C) by Phringer Edgar",
                     dukeIcon,MsgBoxDlg.OKSYSINFO);
      msgBoxDlg.setAutoCancel(inApplet);
      msgBoxDlg.setVisible(true);
    }
    else {
      System.out.println(e.getActionCommand());
    } // end if
  } // end actionPerformed

  public void updateNow() {
    long curTime = System.currentTimeMillis();

    if ((lastGetMessagesQuery+GETMESSAGES_TIME) < curTime) {
      GetMessages(true);
    } // end if
  } // end updateNow

  private void Logout() {
    String errorMsg;
    db.logout();
    errorMsg = db.getErrorMsg();
    if (errorMsg != null) {
      (new ErrorDlg(parent,"Error",!inApplet,errorMsg)).setAutoCancel(inApplet);
    }
    else {
      lModel.setLogedIn(false);
    } // end if
  } // end Logout

  private synchronized boolean GetMessages(boolean onlyNew) {
    // returns true if some output was produced
    if (onlyNew && (!lModel.getLogedIn())) return false;

    Enumeration enum = db.getMessages();
    String errorMsg  = db.getErrorMsg();

    // check for errors
    if (errorMsg != null) {
      (new ErrorDlg(parent,"Error",!inApplet,errorMsg)).setAutoCancel(inApplet);
      return true;
    } // end if

    // assert no errors
    lastGetMessagesQuery = System.currentTimeMillis(); // update time stamp
    if (enum.hasMoreElements()) {
      Vector vector = new Vector();
      boolean match = false;
      int i,n;
      CalendarEntry entry;

      while (enum.hasMoreElements()) vector.addElement(enum.nextElement());
      if (onlyNew) Beep.beep();
      GetMsgDlg getMsgDlg = new GetMsgDlg(parent,"Get Messages",
                                          !inApplet,vector);
      if (inApplet) getMsgDlg.setAutoCancel(true);
      getMsgDlg.setVisible(true);
      // check if new entries are invalidating the scope
      n = vector.size();
      for (i=0; i<n; i++) {
        entry = (CalendarEntry)vector.elementAt(i);
        if (entry.isPeriodic()) {
          dModel.periodicInvalidate();
          return true;
        }
        else if (sModel.matchesScope(entry)) {
          match = true;
        } // end if
      } // end for
      if (match) dModel.invalidate();
      return true;
    } // end if

    // assert enumeration has no elements
    if (!onlyNew) {
      MsgBoxDlg msgBoxDlg = new MsgBoxDlg(parent,"Get Messages",!inApplet,
                     "No new Messages on Server",
                     MsgBoxDlg.OK | MsgBoxDlg.ICONINFORMATION);
      msgBoxDlg.setAutoCancel(inApplet);
      msgBoxDlg.setVisible(true);
      return true;
    } // end if

    // assert no output has been made
    return false;
  } // end GetMessages

  public void itemStateChanged(ItemEvent e) {
    ItemSelectable item = e.getItemSelectable();
    if (item instanceof CheckboxMenuItem) {
      String cmd = ((CheckboxMenuItem)item).getLabel();
      if (cmd.equals("Daily")) {
        //cardLayout.first(cardPanel);
        cardLayout.show(cardPanel,DAYLY_PANEL);
        sModel.setScopeWidth(ScopeModel.DAYLY);
      }
      else if (cmd.equals("Weekly")) {
        //cardLayout.first(cardPanel);
        //cardLayout.next(cardPanel);
        cardLayout.show(cardPanel,WEEKLY_PANEL);
        sModel.setScopeWidth(ScopeModel.WEEKLY);
      }
      else { // if cmd equals monthly
        //cardLayout.first(cardPanel);
        //cardLayout.next(cardPanel);
        //cardLayout.next(cardPanel);
        cardLayout.show(cardPanel,MONTHLY_PANEL);
        sModel.setScopeWidth(ScopeModel.MONTHLY);
      }
    } // end if
  } // end itemStateChanged

} // end JavaNotesGUI

/* ******************************************************************
   end of file
*********************************************************************/
