/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : LoginInfoModel
   date       : 28.12.97
   description: a model for the login state

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.util.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class LoginInfoModel extends Observable {

  // instance variable declarations
  private boolean logedIn;
  private String[] persons;
  private String user;
  private String passwd;
  private int defSelection;
  private boolean logedInChanged = false;
  private boolean personsChanged = false;
  private boolean userChanged    = false;

  // constructors
  public LoginInfoModel() {
    init();
  } // end constructor LoginInfoModel

  // methods
  private void init() {
    logedIn        = false;
    persons        = new String[1];
    persons[0]     = "  < no login >  ";
    user           = "";
    passwd         = "";
    defSelection   = -1;
  } // end init

  public void setLogedIn(boolean logedIn) {
    if (!logedIn) {
      init();
      logedInChanged = true;
      personsChanged = true;
      userChanged    = true;
    }
    else {
      this.logedIn   = logedIn;
      logedInChanged = true;
    } // end if
    checkFlags();
  } // end setLogedIn

  public boolean getLogedIn() {
    return logedIn;
  } // end getLogedIn

  public void setPersons(String[] persons) {
    this.persons   = persons;
    personsChanged = true;
    checkFlags();
  } // end setPersons

  public String[] getPersons() {
    return persons;
  } // end getPersons

  public String[] getPersonsList() {
    // returns the persons list with spezialy highlighted loged in user
    String[] formatedList = new String[persons.length];
    int i;

    for (i=0; i<persons.length; i++) {
      if (persons[i].equals(user)) {
        formatedList[i] = "<"+persons[i]+">";
        defSelection    = i;
      }
      else {
        formatedList[i] = persons[i];
      } // end if
    } // end for
    return formatedList;
  } // end getPersonsList

  public int getDefSelection() {
    // returns the index of the loged in user in the persons list
    int i = 0;

    while ((defSelection < 0) && (i<persons.length)) {
      if (persons[i].equals(user)) defSelection = i;
      i++;
    } // end while
    if (defSelection < 0) defSelection = 0; // if not found
    return defSelection;
  } // end getDefSelection

  public void setUser(String user) {
    this.user   = user;
    userChanged = true;
    checkFlags();
  } // end setUser

  public String getUser() {
    return user;
  } // end getUser

  public void setPasswd(String passwd) {
    this.passwd   = passwd;
  } // end setPasswd

  public String getPasswd() {
    return passwd;
  } // end getPasswd

  private void checkFlags() {
    if (logedInChanged && personsChanged && userChanged) {
      logedInChanged = false;
      personsChanged = false;
      userChanged    = false;
      Assert.out.println("LoginInfoModel: Login changed");
      setChanged();
      notifyObservers();
    } // end if
  } // end checkFlags

} // end LoginInfoModel

/* ******************************************************************
   end of file
*********************************************************************/
