/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : ReplyHeader
   date       : 15.11.97
   description: a header for replies from the java notes server

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.io.*;
import java.util.*;

/* ******************************************************************
   interface
*********************************************************************/

public class ReplyHeader implements Streamable {

  // instance variable declarations
  private String errStr     = "";
  private Vector users;
  private boolean lastInSeq = true;    // indicates that this object is the
                                       // last in a sequence

  // constructors
  public ReplyHeader () {
    // used to reply a request with returns no data and causes no error

    users = new Vector();
  } // end constructor ReplyHeader

  public ReplyHeader (String errorMessage) {
    // used to reply a request with returns no data and causes no error

    if (errorMessage == null)
      errStr = "";
    else
      errStr = errorMessage;
    // end if
    users  = new Vector();
  } // end constructor ReplyHeader

  public ReplyHeader (String[] aStr) {
    // used to reply a request with returns the user list
    int i;

    users  = new Vector();
    for (i=0; i<aStr.length; i++) {
      users.addElement(aStr[i]);
    } // end for
  } // end constructor ReplyHeader

  public ReplyHeader (DataInput in) throws IOException {
    // used to create a ReplyHeader instance from a stream

    users = new Vector();
    inputFrom(in);
  } // end constructor ReplyHeader

  // methods
  public void outputTo(DataOutput out) throws IOException {
    int n = users.size();
    int i;

    out.writeUTF(errStr);
    for (i=0; i<n; i++) out.writeUTF((String)users.elementAt(i));
    out.writeUTF("");
    out.writeBoolean(lastInSeq);
  } // end outputTo

  public void inputFrom(DataInput in) throws IOException {
   String str;

    errStr    = in.readUTF();
    users.removeAllElements();
    str = in.readUTF();
    while (!str.equals("")) {
      users.addElement(str);
      str = in.readUTF();
    } // end while;
    lastInSeq = in.readBoolean();
  } // end inputFrom

  public void setLastInSeq(boolean lastInSeq) {
  // sets if object is the last in a sequence or not

    this.lastInSeq = lastInSeq;
  } // end setLastInSeq

  public boolean isLastInSeq() {
  // indicates if object is the last in a sequence or not

    return lastInSeq;
  } // end isLastInSeq

  public boolean failed() {
  // returns true if the last request failed

    return !errStr.equals("");
  } // end failed

  public String getErrorMsg() {
  // returns a string describing the last error

    return errStr;
  } // end getErrorMsg

  public String[] getUsers() {
    int n = users.size();
    int i;
    String[] aStr = new String[n];

    for (i=0; i<n; i++) aStr[i] = (String)users.elementAt(i);
    return aStr;
  } // end getUsers

} // end ReplyHeader

/* ******************************************************************
   end of file
*********************************************************************/
